/** Process related interface functions.
 *  @file libaragorn.c
 *  
 *  Copyright (C) 2001-2006, INRIA, Universite de Rennes 1, EDF.
 */

#include <stdio.h>
#include <sys/syscall.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>

#include <kerrighed/types.h>
#include <kerrighed/krgnodemask.h>
#include <epm/migration_types.h>
/* #include <proc/krg_fork_types.h> */
/* #include <kerrighed/proc/process_group_types.h> */

#include <checkpoint.h>
#include <kerrighed_tools.h>


#define KRG_SERVICES_PATH "/proc/kerrighed/services"

//#define DEBUG_LIBARAGORN

extern pid_t __libc_fork();



/*****************************************************************************/
/*                                                                           */
/*                              INTERNAL FUNCTIONS                           */
/*                                                                           */
/*****************************************************************************/



#if 0 /* Deprecated */
/** get the thread id */

static int get_thread_id(void)
{
  int res, thread_id ;

  res = call_kerrighed_services(KSYS_GET_KRG_THREAD_ID, &thread_id);

  if ( res >= 0)
    return thread_id ;
  else
    return -EPERM ;
}
#endif


/*****************************************************************************/
/*                                                                           */
/*                              EXPORTED FUNCTIONS                           */
/*                                                                           */
/*****************************************************************************/



/* Return the node id of the local machine */

int get_cpu_id (void)
{
  int res;
  int node_id;

  res = call_kerrighed_services(KSYS_GET_NODE_ID, &node_id);

  if (res != 0)
    return -1;

  return node_id;
}


/* Return the number of nodes in the cluster */

int get_nr_cpu (void)
{
  int res;
  int count;

  res = call_kerrighed_services(KSYS_GET_NODES_COUNT, &count);

  if (res != 0)
    return -1;

  return count;
}


/** Migrate a process.
 *  @author Geoffroy Valle
 *
 *  @param pid          Pid of process to migrate.
 *  @param destination  Node ID of the destination node.
 */
int migrate (pid_t pid, int destination_node)
{
  int res;
  migration_infos_t migration_infos;

  migration_infos.process_to_migrate = pid;
  migration_infos.destination_node_id = destination_node;

  res = call_kerrighed_services(KSYS_PROCESS_MIGRATION,  &migration_infos);

  return res;
}



#if 0 /* Code disabled */
/** Remove a node from a node group
 * @author Geoffroy Valle
 *
 * @param nid        Node identifier
 * @param groupId    Group identifier
 */
int remove_node_from_group (kerrighed_node_t nid, node_group_id_t groupId)
{
  int res;
  node_group_add_info_t info;

  info.nid = nid ;
  info.groupId = groupId;

  res = call_kerrighed_services(KSYS_REMOVE_NODE_FROM_PROC_GROUP, &info);

  return res;
}



/** Add a node from a node group
 * The group identifier is defined by the system administrator
 * @author Geoffroy Valle
 *
 * @param nid        Node identifier
 * @param groupId    Group identifier
 */
int add_node_in_group (kerrighed_node_t nid, process_group_id_t groupId)
{
  int res;
  node_group_add_info_t info;

  info.nid = nid;
  info.groupId = groupId;

  res = call_kerrighed_services(KSYS_ADD_NODE_IN_PROC_GROUP, &info);

  return res;
}



/** Remove a process from a process group
 * @author Geoffroy Valle
 *
 * @param pid        process identifier
 * @param groupId    Group identifier
 */
int remove_process_from_group (pid_t pid, process_group_id_t groupId)
{
  int res;
  process_group_add_info_t info;

  info.pid = pid;
  info.tid = 0 ;
  info.groupId = groupId;

  res = call_kerrighed_services(KSYS_REMOVE_PROCESS_FROM_PROC_GROUP, &info);

  return res;
}



/** Add a process from a process group
 * The group identifier is defined by the system administrator
 * @author Geoffroy Valle
 *
 * @param pid        Process identifier
 * @param groupId    Group identifier
 */
int add_process_in_group (pid_t pid, process_group_id_t groupId)
{
  int res;
  process_group_add_info_t info;

  info.pid = pid;
  info.tid = 0 ;
  info.groupId = groupId;

  res = call_kerrighed_services(KSYS_ADD_PROCESS_IN_PROC_GROUP, &info);

  return res ; 
}

process_group_id_t get_group(void) 
{
  int res ;
  process_group_id_t result ;

  res = call_kerrighed_services(KSYS_GET_PROC_GROUP, &result);

  if ( res >= 0)
    return result ;
  else
    return 0 ;
}

process_group_id_t create_group(void) 
{
  int res ;
  process_group_id_t result ;

  res = call_kerrighed_services(KSYS_CREATE_PROC_GROUP, &result);

  if ( res >= 0)
    return result ;
  else
    return 0 ;
}



int delete_group(process_group_id_t groupId)
{
  int res ;

  res = call_kerrighed_services(KSYS_DELETE_PROC_GROUP, &groupId);

  return res ;
}
#endif /* 0 */


int migrate_self (int destination_node)
{
  int res;
  migration_infos_t migration_infos;

  migration_infos.process_to_migrate = getpid();
  migration_infos.destination_node_id = destination_node;

  res = call_kerrighed_services(KSYS_PROCESS_MIGRATION, &migration_infos);

  return res ;
}



/** Make a node leave the cluster
 *  @author Renaud Lottiaux
 *
 */
int krg_leave(void)
{
  int res;
  int dummy;

  res = call_kerrighed_services(KSYS_LEAVE_CLUSTER, &dummy);

  return res;
}



/** Make a node join the cluster
 *  @author Renaud Lottiaux
 *
 */
int krg_join(void)
{
  int res;
  int dummy;

  res = call_kerrighed_services(KSYS_JOIN_CLUSTER, &dummy);

  return res;
}


checkpoint_infos_t application_checkpoint_from_appid (media_t media, 
						      long app_id, int signal)
{
  checkpoint_infos_t ckpt_infos;

  ckpt_infos.app_id = app_id;
  ckpt_infos.chkpt_sn = 0;
  ckpt_infos.type = FROM_APPID;
  ckpt_infos.media = media;
  ckpt_infos.signal = signal;
  ckpt_infos.result = call_kerrighed_services(KSYS_APP_CHKPT, &ckpt_infos);

  return ckpt_infos;
}

checkpoint_infos_t application_checkpoint_from_pid (media_t media, 
						    pid_t pid, int signal)
{
  checkpoint_infos_t ckpt_infos;

  ckpt_infos.app_id = pid;
  ckpt_infos.chkpt_sn = 0;
  ckpt_infos.type = FROM_PID;
  ckpt_infos.media = media;
  ckpt_infos.signal = signal;
  ckpt_infos.result = call_kerrighed_services(KSYS_APP_CHKPT, &ckpt_infos);

  return ckpt_infos;
}

checkpoint_infos_t inside_app_checkpoint(void)
{
  checkpoint_infos_t ckpt_infos;

  ckpt_infos.app_id = getpid();
  ckpt_infos.chkpt_sn = 0;
  ckpt_infos.type = FROM_INSIDE;
  ckpt_infos.media = DISK;
  ckpt_infos.result = call_kerrighed_services(KSYS_APP_CHKPT, &ckpt_infos);

  return ckpt_infos;
}

int application_restart(media_t media, long app_id, int chkpt_sn, int flags)
{
  restart_request_t rst_req;

  rst_req.app_id = app_id;
  rst_req.chkpt_sn = chkpt_sn;
  rst_req.media = media;
  rst_req.flags = flags;

  return  call_kerrighed_services(KSYS_APP_RESTART,
				  &rst_req);
}


/*****************************************************************************/
/*                                                                           */
/*                        KERRIGHED DEVELOPERS FUNCTIONS                     */
/*                                                                           */
/*****************************************************************************/


#if 0
int krg_thread_remove_from_group (int thread_id, process_group_id_t groupId)
{
  int res;
  process_group_add_info_t info;

  info.pid = getpid() ;
  info.tid = thread_id ;
  info.groupId = groupId;

  res = call_kerrighed_services(KSYS_REMOVE_PROCESS_FROM_PROC_GROUP, &info);

  return res;
}

int krg_thread_add_in_group (int thread_id, process_group_id_t groupId)
{
  int res;
  process_group_add_info_t info;

  info.pid = getpid() ;
  info.tid = thread_id ;
  info.groupId = groupId ;

  res = call_kerrighed_services(KSYS_ADD_PROCESS_IN_PROC_GROUP, &info) ;

  return res;

}
#endif /* 0 */


int thread_migrate (int thread_id, int destination_node)
{
  int res;
  migration_infos_t migration_infos;

  migration_infos.thread_to_migrate = thread_id ;
  migration_infos.destination_node_id = destination_node;

  res = call_kerrighed_services(KSYS_THREAD_MIGRATION, &migration_infos);

  return res;
}
