/*
 *  Copyright (C) 2001-2006, INRIA, Universite de Rennes 1, EDF.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <getopt.h>
#include <time.h>
#include <kerrighed.h>

#define CHKPT_DIR "/var/chkpt/"

int main(int argc, char *argv[])
{
	int r = 0;
	int pid = -1;
	short from_appid = 0;
	media_t media = DISK;
	int signal = 0;

	char * description = NULL;

	// is Kerrighed launched ?
	if (get_nr_cpu() == -1)
	{
		printf ("%s: no kerrighed nodes found\n", argv[0]);
		exit(-1);
	}

	// Manage options with getopt
	{
		char c;
		int option_index = 0;
		char * short_options= "ham:d:k::";
		static struct option long_options[] =
			{
				{"help", no_argument, 0, 'h'},
				{"from-appid", no_argument, 0, 'a'},
				{"media", required_argument, 0, 'm'},
				{"description", required_argument, 0, 'd'},
				{"kill", optional_argument, 0, 'k'},
				{0, 0, 0, 0}
			};

		while ((c = getopt_long(argc, argv, short_options,
					long_options, &option_index)) != -1) {
			switch (c) {
			case 'h':
				printf("Usage : checkpoint [-h] "
				       "[-k | --kill | -kSIGNAL | --kill=SIGNAL] "
				       "[a|--from-appid] "
				       "[(-m|--media) disk|memory] "
				       "[(-d|--description) description] "
				       "pid \n");
				return 0;
				break;
			case 'a':
				from_appid=1;
				break;
			case 'm':
				if (strcmp(optarg, "DISK") == 0)
					media = DISK;
				else if (strcmp(optarg, "MEMORY") == 0)
					media = MEMORY;
				else printf("Warning: Unknown type of media\n");
				break;
			case 'd':
				description = optarg;
				break;
			case 'k':
				if (optarg)
					signal = atoi(optarg);
				else 
					signal = 15;
				break;
			default:
				printf("Warning: Option %c "
				       "not implemented\n", c);
				break;
			}
		}
	}

	pid = atoi( argv[argc-1] );
	if (pid < 2) return -1;

	checkpoint_infos_t infos;
	if (from_appid) {
		printf("Checkpointing application %d\n", pid);
		infos = application_checkpoint_from_appid(media, pid, signal);
	} else {
		printf("Checkpointing application in which "
		       "process %d is involved\n", pid);
		infos = application_checkpoint_from_pid(media, pid, signal);
	}

	r = infos.result;

	if (r != 0) goto exit;

	if (description == NULL) description = "No description";

	time_t date = time(NULL);

	printf("Identifier: %ld\n"
	       "Version: %d\n"
	       "Description: %s\n"
	       "Date: %s\n",
	       infos.app_id,
	       infos.chkpt_sn,
	       description,
	       ctime(&date)
		);

	// need to write it in a file
	FILE* fd;

	char path[256];
	sprintf(path, "%s/%ld/description_v%d.txt", CHKPT_DIR,
		infos.app_id, infos.chkpt_sn);

	if ((fd = fopen(path, "a")) == NULL)  {
		perror(path);
		exit(1);
	} else {
		fprintf(fd,
			"Identifier: %ld\n"
			"Version: %d\n"
			"Description: %s\n"
			"Date: %ld\n",
			infos.app_id,
			infos.chkpt_sn,
			description,
			date);
		fclose(fd);
	}

exit:

	if (r != 0) {
		r = errno;
		perror("Checkpoint");
	}

	return r;
}
