/*
 *  Copyright (C) 2001-2006, INRIA, Universite de Rennes 1, EDF.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

#include <kerrighed.h>

#define CAP_DEF(cap) [CAP_##cap] = #cap

const char * cap_text[CAP_SIZE] = {
  CAP_DEF(CHANGE_KERRIGHED_CAP),
  CAP_DEF(CAN_MIGRATE),
  CAP_DEF(DISTANT_FORK),
  CAP_DEF(FORK_DELAY),
  CAP_DEF(CHECKPOINTABLE),
  CAP_DEF(USE_REMOTE_MEMORY),
  CAP_DEF(USE_INTRA_CLUSTER_KERSTREAMS),
  CAP_DEF(USE_INTER_CLUSTER_KERSTREAMS),
  CAP_DEF(USE_WORLD_VISIBLE_KERSTREAMS),
  CAP_DEF(SEE_LOCAL_PROC_STAT),
  CAP_DEF(DEBUG)
};

int check_version(void)
{
	/* Should test if all kernel capabilities are known... */
	return 1;
}

int print_capability(int current_cap_vector)
{
  int i ;
  int char_printed_on_this_line = 8 ;
  int char_to_print = 0 ;
  int lines_printed = 0 ;
  for (i=0; i < CAP_SIZE ;i++)
    {
      if (cap_raised(current_cap_vector, i))
	{
	  char_to_print = strlen(cap_text[i])+2;
	  if (char_printed_on_this_line+char_to_print < 80)
	    {
	      if (char_printed_on_this_line!=8)
		{
		  printf(", ") ;
		  char_to_print += 2 ;
		}
	      else
		printf("\t") ;
	    }
	  else
	    {
	      printf("\n\t") ;
	      lines_printed++ ;
	      char_printed_on_this_line = 8 ;
	    }
	  printf("%s",cap_text[i]) ;
	  char_printed_on_this_line+=char_to_print ;
	}
    }
  printf("\n") ;
  return 0 ;
}


int print_capabilities(krg_cap_t * current_caps)
{
  printf("Permitted Capabilities: 0%o\n", krg_cap_getpermitted(current_caps));
  print_capability(krg_cap_getpermitted(current_caps)) ;

  printf("Effective Capabilities: 0%o\n", krg_cap_geteffective(current_caps));
  print_capability(krg_cap_geteffective(current_caps)) ;

  printf("Inheritable Permitted Capabilities: 0%o\n", krg_cap_getinheritable_permitted(current_caps));
  print_capability(krg_cap_getinheritable_permitted(current_caps)) ;

  printf("Inheritable Effective Capabilities: 0%o\n", krg_cap_getinheritable_effective(current_caps));
  print_capability(krg_cap_getinheritable_effective(current_caps)) ;

  return 0 ;
}



int construct_capability(char * description)
{
  int res = 0;
  int text_matched ;
  int i ;
  char ** next_token = &description ;
  char * current_token ;
  while (*next_token)
    {
      current_token = strsep (next_token, ",") ;
      text_matched = 0 ;
      for (i=0; i < CAP_SIZE ;i++)
	{
	  if (!strncmp(cap_text[i],current_token, strlen(cap_text[i])))
	    {
	      res |= 1<<i ;
	      text_matched = 1 ;
	      break ;
	    }
	}
      if (!text_matched)
	{
	  printf("Unrecognized capability: %s\n", current_token) ;
	  printf("Authorized capability names are:\n") ;
	  print_capability(~0) ;
	}
    }
  return res ;
}



int update_capability(char* description, int oldcapability)
{
  int res, tmp ;
  
  switch (description[0])
    {
    case '+' :
      tmp = construct_capability(&description[1]) ;
      res = tmp | oldcapability ;
      break ;
    case '-' :
      tmp = construct_capability(&description[1]) ;
      res = ~tmp & oldcapability ;
      break ;
    default:
      res = construct_capability(description) ;
    }
  return res ;
}

int print_usage(char * argv[])
{
  printf("Usage: \n\t%s -h : show this help\n", argv[0]) ;
  printf("\t%s [-k <pid>] -s : show the capabilities of the calling (or designated) process\n", argv[0]) ;
  printf("\t%s -f : do not ask for confirmation when removing CHANGE_KERRIGHED_CAP\n", argv[0]) ;
  printf("\t%s [-k <pid>] {-[EPID] <num> }* : set Effective, Permitted, Inheritable_Permitted,\n\t\tInheritable_Effective (Default effective of son) capabilities of this (or designated) process\n", argv[0]) ;
  printf("\t%s [-k <pid>] {-[epid] [+|-] <cap_list> }* : set Effective, Permitted, Inheritable_Permitted,\n\t\tInheritable_Effective (Default effective of son) capabilities of this (or designated) process\n", argv[0]) ;
  printf("Where cap list is one of: \n") ;
  print_capability(~0) ;
  return 0 ;
}

int force = 0 ;
int changed_father_cap = 0 ; 
int main (int argc, char * argv[])
{
  pid_t pid = -1 ;
  int c ;
  int res ;
  char * ignored = NULL ;
  krg_cap_t initial_caps ;

  if (!check_version())
    {
      printf("This will not work : program needs update\n") ;
      return -1 ;
    }

  res = krg_father_capget(&initial_caps) ;
  
  if (res)
    {
      printf ("Unable to get my father's capabilities res: %d, errno: %d\n", res, errno) ;
      goto end ;
    }

  //don't manage inheritable_effective at first
  //initial_caps.krg_cap_inheritable_effective = ~0 ;

  while (1)
    {
      c = getopt(argc, argv, "P:E:I:D:p:e:i:d:shfk:");
      if (c == -1)
	break;
      switch (c) {
      case 'f':
	force = 1 ;
	break;
      case 'h':
	print_usage(argv) ;
	break;
      case 's':
	print_capabilities(&initial_caps) ;
	break;
      case 'e':
	initial_caps.krg_cap_effective = update_capability(optarg, initial_caps.krg_cap_effective) ;
	changed_father_cap = 1 ;
	break;
      case 'E':
	initial_caps.krg_cap_effective = strtol(optarg,&ignored,0) ;
	changed_father_cap = 1 ;
	break;
      case 'p':
	initial_caps.krg_cap_permitted = update_capability(optarg, initial_caps.krg_cap_permitted) ;
	changed_father_cap = 1 ;
	break;
      case 'P':
	initial_caps.krg_cap_permitted = strtol(optarg,&ignored,0) ;
	changed_father_cap = 1 ;
	break;
      case 'i':
	initial_caps.krg_cap_inheritable_permitted = update_capability(optarg, initial_caps.krg_cap_inheritable_permitted) ;
	changed_father_cap = 1 ;
	break;
      case 'I':
	initial_caps.krg_cap_inheritable_permitted = strtol(optarg,&ignored,0) ;
	changed_father_cap = 1 ;
	break;
      case 'd':
	initial_caps.krg_cap_inheritable_effective = update_capability(optarg, initial_caps.krg_cap_inheritable_effective) ;
	changed_father_cap = 1 ;
	break;
      case 'D':
	initial_caps.krg_cap_inheritable_effective = strtol(optarg,&ignored,0) ;
	changed_father_cap = 1 ;
	break;
      case 'k':
	pid = atoi (optarg) ;
	if (changed_father_cap)
	  {
	    printf("Conflict in implementation: -k must be the first option used\n") ;
	    goto end ;
	  }
	res = krg_pid_capget(pid, &initial_caps) ;
	if (res)
	  {
	    printf ("Unable to get capabilities of process %d (res: %d, errno: %d))\n", pid, res, errno) ;
	    goto end ;
	  }
	break;
      }
    }
  if (changed_father_cap && (!force && (!cap_raised(initial_caps.krg_cap_permitted, CAP_CHANGE_KERRIGHED_CAP) ||
				       !cap_raised(initial_caps.krg_cap_effective, CAP_CHANGE_KERRIGHED_CAP) )))
    {
      int answer ;
      int use_default = 1 ;
      printf("Are you sure you want to loose the capability to change your capabilities (Y/n)\n") ;
      answer = getchar() ;
      while (answer != 'y' && answer != 'n' && answer != EOF && answer != '\n' && answer != 'Y')
	{
	  use_default = 0 ;
	  answer = getchar() ;
	}
      if (answer != 'y' && answer != 'Y' && (answer != '\n' || !use_default) )
	exit(0) ;

    } ;
  if (changed_father_cap)
    {
      if (pid == -1)
	{
	  res = krg_father_capset(&initial_caps) ;
	  if (res)
	    printf ("Unable to set my father's capabilities\n") ;
	}
      else
	{
	  res = krg_pid_capset(pid, &initial_caps) ;
	  if (res)
	    printf ("Unable to set the capabilities of process %d (res is %d, errno %d)\n", pid, res, errno) ;
	}
    }

 end:
  return res ;
}
