/*
 *  Copyright (C) 2001-2006, INRIA, Universite de Rennes 1, EDF.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <getopt.h>
#include <time.h>
#include <kerrighed.h>

#define CHKPT_DIR "/var/chkpt/"

long appid ;
int version ;
media_t media;
int flags = 0;

void show_help()
{
	printf ("Restart an application\nusage: restart [-h]"
		" [--media DISK|MEMORY] id version\n");
	printf ("  -h : This help\n");
	printf ("  -p : to restart a single process. "
		"In this case, version is ignored.\n");
	exit(1);
}

void parse_args(int argc, char *argv[])
{
	char c;
	int option_index = 0;
	char * short_options= "hm:p";
	static struct option long_options[] =
		{
			{"help", no_argument, 0, 'h'},
			{"media", required_argument, 0, 'm'},
			{"pid", no_argument, 0, 'p'},
			{0, 0, 0, 0}
		};
    
	while ((c = getopt_long(argc, argv, short_options,
				long_options, &option_index)) != -1) {
		switch (c)
		{
		case 'h':
			goto err;
			break;
		case 'm':
			if (strcmp(optarg, "DISK") == 0) 
				media = DISK;
			else if (strcmp(optarg, "MEMORY") == 0) 
				media = MEMORY;
			else printf("Warning: Unknown type of media\n");
			break;
		case 'p':
			flags = RESTART_PID_ONLY;
			break;
		default:
			printf("** unknown option\n");
		}
	}

	if (argc - optind < 2)
		goto err;

	appid = atoi(argv[argc-2]);
	version = atoi(argv[argc-1]);
  
	return ;
  
err:
	show_help();
}

void show_gui()
{  
	show_help();
/*   printf("Identifier Version  Date" */
/* 	 "                       Description\n"); */
/*   printf("----------------------------------------" */
/* 	 "----------------------------------------\n"); */

/*   FILE* fd; */
  
/*   if ((fd = fopen(CHKPT_DIR, "r")) == NULL)  { */
/*     perror(CHKPT_LIST);  */
/*     exit(1); */
/*   }  */
/*   time_t date; */
/*   char description[256] = ""; */
  
/*   printf("Identifier Version  Date" */
/* 	 "                       Description\n"); */
/*   printf("----------------------------------------" */
/* 	 "----------------------------------------\n"); */

/*   while ( fscanf(fd,"%d;%d;%ld;%[ a-zA-Z0-9_,!:*$-];\n", &pid, &version, &date, */
/* 		 description) == 4 ){ */
/*     printf("%10d %5d    %.24s   %-.25s\n", pid, version, ctime(&date), description); */
/*   } */
/*   fclose(fd); */

/*   printf("----------------------------------------" */
/* 	 "----------------------------------------\n"); */
/*   printf("Which application to restart : "); scanf("%d", &pid); */
/*   printf("                     version : "); scanf("%d", &version);   */
/*   printf("----------------------------------------" */
/* 	 "----------------------------------------\n"); */
}

int main(int argc, char *argv[])
{
	int r = 0 ;

	if (get_nr_cpu() == -1)
	{
		printf ("%s: no kerrighed nodes found\n", argv[0]);
		exit(-1);
	}

	if (argc > 1) {
		parse_args(argc, argv);
	} else {
		show_gui();
	}

	if (flags != RESTART_PID_ONLY)
	{  
		int res = -1;
		char name[120];
		struct stat buf;

		snprintf (name, 120, "/var/chkpt/%ld/global_v%d.bin", 
			  appid, version);
    
		res = stat(name, &buf);
    
		if (res < 0)
		{
			fprintf (stderr, "%s: %ld: no such process\n", argv[0], appid);
			fprintf (stderr, "File not found : %s\n", name);
			exit (-1);
		}
	}

	printf("Restart application %ld (v%d) ...\n", appid, version);

	r = application_restart (media, appid, version, flags);
	if (r != 0)
		r = errno;
	
	switch (r) {
	case 0:
		break;
	case E_CR_APPBUSY:
		fprintf(stderr, "restart: an application using appid %ld is already "
		       "running\n", appid);
		break;
	case E_CR_PIDBUSY:
		fprintf(stderr, "restart: one PID of the application is already in use");
		break;
	default:
		perror("restart");
	}

	return r;
}
